"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var fs = require('fs');
var path = require('path');
var ramda_1 = __importDefault(require("ramda"));
var acorn_1 = require("./acorn");
var js_gen_1 = require("./js-gen");
var node_ops_1 = require("./node-ops");
var redux_1 = require("./redux");
var PropStyle;
(function (PropStyle) {
    PropStyle[PropStyle["Input"] = 0] = "Input";
    PropStyle[PropStyle["ReduxFunc"] = 1] = "ReduxFunc";
    PropStyle[PropStyle["State"] = 2] = "State";
})(PropStyle = exports.PropStyle || (exports.PropStyle = {}));
var propListToMap = function (props) {
    var result = new Map();
    props.forEach(function (p) {
        var key = ramda_1.default.path(['key', 'name'], p);
        if (key !== undefined) {
            var val = ramda_1.default.path(['value', 'raw'], p);
            result.set(key, val);
        }
    });
    return result;
};
exports.extractProp = function (propNode, propDefaults, redux) {
    var val = propNode.value;
    var name = ramda_1.default.path(['key', 'name'], propNode);
    var defaultValue = propDefaults.get(name);
    var propType = '?';
    var optional = false;
    if (val.type === 'CallExpression') {
        propType = js_gen_1.genJs(val);
    }
    else {
        if (ramda_1.default.pathEq(['object', 'type'], 'Identifier', val)) {
            propType = ramda_1.default.path(['property', 'name'], val);
            optional = true;
        }
        else {
            propType = ramda_1.default.path(['object', 'property', 'name'], val);
        }
    }
    var style = PropStyle.Input;
    if (redux.funcNames.some(ramda_1.default.equals(name))) {
        style = PropStyle.ReduxFunc;
    }
    else if (redux.returnedPropNames.some(ramda_1.default.equals(name))) {
        style = PropStyle.State;
    }
    return {
        default: defaultValue,
        name: name,
        optional: optional,
        style: style,
        type: propType,
    };
};
var extractComponent = function (ast) {
    if (ast === undefined) {
        return;
    }
    var componentClass = node_ops_1.searchBySuperClass(ast, 'Component');
    if (componentClass !== undefined) {
        var className = componentClass.id.name;
        var propDefaults_1 = propListToMap(node_ops_1.searchForPropDefaults(ast, className));
        var redux_2 = redux_1.getReduxDetails(ast);
        var props = node_ops_1.searchForPropTypes(ast, className)
            .map(function (prop) { return exports.extractProp(prop, propDefaults_1, redux_2); })
            .sort(function (a, b) { return a.name.localeCompare(b.name); });
        return {
            componentName: className,
            definition: ast,
            props: props,
            redux: redux_2,
        };
    }
};
var processFile = function (filePath) {
    return new Promise(function (resolve, reject) {
        fs.readFile(filePath, function (err, content) {
            if (err) {
                reject(err);
            }
            else {
                var ast = void 0;
                var component = void 0;
                try {
                    ast = acorn_1.parse(content);
                    component = extractComponent(ast);
                }
                catch (e) {
                    // console.log(`Could not parse ${filePath}: ${e}`);
                }
                resolve({
                    ast: ast,
                    component: component,
                    content: content,
                    fileName: path.basename(filePath),
                    filePath: filePath,
                });
            }
        });
    });
};
var SourceFileCache = /** @class */ (function () {
    function SourceFileCache() {
        this.data = new Map();
    }
    SourceFileCache.prototype.getFile = function (filePath) {
        if (this.data.has(filePath)) {
            return this.data.get(filePath);
        }
        else {
            var parsePromise = processFile(filePath);
            this.data.set(filePath, parsePromise);
            return parsePromise;
        }
    };
    SourceFileCache.prototype.fileNames = function () {
        return Array.from(this.data.keys());
    };
    SourceFileCache.prototype.cachedFiles = function () {
        return Array.from(this.data.values());
    };
    SourceFileCache.prototype.refresh = function (filePath) {
        var parsePromise = processFile(filePath);
        this.data.set(filePath, parsePromise);
    };
    SourceFileCache.prototype.clear = function (filePath) {
        this.data.delete(filePath);
    };
    return SourceFileCache;
}());
exports.SourceFileCache = SourceFileCache;
//# sourceMappingURL=tree-view.js.map